@@ -1,6 +1,6 @@
 package Provision::Unix::VirtualOS::FreeBSD::Ezjail;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use warnings;
 use strict;
@@ -72,6 +72,10 @@ sub create_virtualos {
 }
 
 sub is_present {
+    my $self = shift;
+    my $homedir = $self->get_ve_home();
+    return $homedir if -d $homedir;
+    return;
 };
 
 sub get_console {
@@ -81,6 +85,12 @@ sub get_console {
     exec "$cmd console $ctid";
 };
 
+sub get_ve_home {
+    my $self = shift;
+    my $ctid = $vos->{name};
+    return "/usr/jails/$vos->{name}";
+};
+
 sub _get_jails_root {
     my $r = `grep '^ezjail_jaildir' /usr/local/etc/ezjail.conf`;
     if ($r) {
